chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  // 登录
  if(request.action === 'redirect_gpt') {
    let id = chrome.runtime.id
    let url = "chrome-extension://" + id + "/views/CaptainGPT/dist/index.html#/?type='gptSingleWindow'"
    chrome.tabs.update(sender.tab.id, {url: url});
  }

  // 从页面打开GPT窗口
  if (request.message === "open window from page") {
    // 使用chrome.storage.local来传递prompt参数
    if (Reflect.has(request, 'prompt') && request.prompt) {
      chrome.storage.local.set({
        initParams: {
          noContextualizing: 1,
          noImmediatelySend: 1,
          input: request.prompt
        }
      });
    } else {
      chrome.storage.local.set({initParams: ''});
    }
    chrome.storage.local.get(["gptWindowId"], (result) => {
      // 使用缓存的窗口ID
      if (result && result.gptWindowId) {
        chrome.windows.get(result.gptWindowId, (window) => {
          if (chrome.runtime.lastError) {
            chrome.storage.local.remove(["gptWindowId"]);
            openpage();
          } else {
            chrome.windows.update(result.gptWindowId, { focused: true }, (res) => {
              if (!res) {
                openpage();
              }
            });
          }
        });
      } else {
        openpage();
      }
    });
    // 打开CaptainGPT窗口
    function openpage() {
      // 默认宽高和位置
      let newWindowWidth = 1360;
      let newWindowHeight = 700;
      let newWindowLeft = 1360;
      let newWindowTop = 0;
      
      chrome.windows && chrome.windows.create({
          url: "../views/CaptainGPT/dist/index.html?type='gptSingleWindow'", // 一个Vue项目打包后的入口文件
          type: "popup",
          width: newWindowWidth,
          height: newWindowHeight,
          left: newWindowLeft,
          top: newWindowTop
      }, (createdWindow) => {
          // 聚焦到新窗口
          chrome.windows.update(createdWindow.id, { focused: true });
          // 将窗口ID保存到chrome.storage中
          chrome.storage.local.set({ gptWindowId: createdWindow.id });
      });
    }
    sendResponse(chrome.windows);
  }

  // 判断是否收到关闭插件的消息
  if (request.action === "closePlugin") {
    chrome.storage.local.get(["openFromPage"], (result) => {
      if (result && result.openFromPage) {
        if (result.openFromPage == 1) {
          // 通知当前tab，关闭CaptainGPT抽屉
          chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
            if (tabs && tabs.length > 0) {
              const currentTab = tabs[0];
              chrome.tabs.sendMessage(currentTab.id, { message: 'close CaptainGPT Window' });
            }
          });
        } else if (result.openFromPage == 2) {
          closePopupWindow()
        }
      }
    });

    function closePopupWindow() {
      chrome.storage.local.get(["gptWindowId"], (result) => {
        // 使用缓存的窗口ID
        if (result && result.gptWindowId) {
          chrome.windows.get(result.gptWindowId, (window) => {
            if (chrome.runtime.lastError) {
              chrome.storage.local.remove(["gptWindowId"]);
              chrome.windows.getCurrent((window) => {
                chrome.windows.remove(window.id, function() {
                  // 在窗口关闭后执行的操作
                });
              });
            } else {
              chrome.windows.remove(result.gptWindowId, function() {
                // 在窗口关闭后执行的操作
              });
            }
          });
        }
      });
    }
  }

  if (request.action === "goToCaptainGPTPage") {
    chrome.storage.local.get(["openFromPage"], (result) => {
      if (result && result.openFromPage) {
        if (result.openFromPage == 1) {
          chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
            if (tabs && tabs.length > 0) {
              const currentTab = tabs[0];
              chrome.tabs.sendMessage(currentTab.id, { message: 'login CaptainGPT from Page' });
            }
          });
        }
      }
    });
  }

  if (request.action === "login_gpt_by_code") {
    let url = 'https://www.captainbi.com/public_Az_get_code.php'
    if (request.params && request.params.is_force==1) {
      url = url + '?is_force=1'
    }
    fetch(url, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json'
      }
    })
    .then(response => response.json())
    .then(res => {
      // 把 结果返回给 content_script.js
      chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        if (tabs && tabs.length > 0) {
          const currentTab = tabs[0];
          chrome.tabs.sendMessage(currentTab.id, { message: 'login_gpt_by_code_background', res: res });
        }
      });
    })
    .catch(error => {
      // 处理请求失败的情况
      console.error(error);
    });
  }
});

// 监听菜单项点击事件
chrome.contextMenus.onClicked.addListener((info, tab) => {
  chrome.tabs.sendMessage(tab.id, { message: 'CaptainGPT ContextMenus Action', menuItemId: info.menuItemId, selectedText: info.selectionText});
});
// 在扩展的background脚本中
chrome.runtime.onInstalled.addListener(function() {
  chrome.contextMenus.create({
    "id": "captaingpt_en_translate",
    "title": "翻译成英文",
    "contexts": ["selection"] // all 右键菜单都会显示，page 仅页面菜单会显示，selection 仅选中文本右键菜单显示，link 链接右键菜单显示，image 图片右键菜单显示
  });
  chrome.contextMenus.create({
      "id": "captaingpt_zh_translate",
      "title": "翻译成中文",
      "contexts": ["selection"]
  });
  chrome.contextMenus.create({
    "id": "captaingpt_msg_reply",
    "title": "生成消息答复",
    "contexts": ["selection"]
  });
  chrome.contextMenus.create({
    "id": "captaingpt_optimizing_reply",
    "title": "优化消息答复",
    "contexts": ["selection"]
  });
});